# This tries to push the hilbert transform for ssb demod back into the
# freq. xlating filter.
#
# The usual gnuradio copyright notice is hereby included by reference.
#
# The starting point for this was weaver_isb_am1_usrp3.py.
#
# The tap coefficients for freq_xlating_fir_filter_ccf were generated
# externally and are read from a file because I didn't want to learn how
# to make fir filters with arbitrary phase response using python numeric
# facilities.
#
# They were generated using Scilab which I am already familiar with.


from gnuradio import gr, gru
from gnuradio import audio
from gnuradio import usrp

class ssb_demod( gr.hier_block ):
    def __init__( self, fg, if_rate, af_rate ):

        self.if_rate  = if_rate
        self.af_rate  = af_rate
        self.if_decim = if_rate / af_rate
        self.sideband = 1

        self.xlate_taps = ([complex(v) for v in file('f').readlines()])
        
        self.audio_taps = gr.firdes.low_pass(
            1.0,
            self.af_rate,
            3e3,
            600,
            gr.firdes.WIN_HAMMING )

        self.xlate = gr.freq_xlating_fir_filter_ccf(
            self.if_decim,
            self.xlate_taps,
            0,
            self.if_rate )

        self.split = gr.complex_to_float( )
        self.split2 = gr.complex_to_float( )

        self.lpf_i = gr.fir_filter_fff(
            1, self.audio_taps )

        self.lpf_q = gr.fir_filter_fff(
            1, self.audio_taps )

        self.sum_l = gr.add_ff( )
        self.sum_u = gr.sub_ff( )
        self.sel_u = gr.multiply_const_ff( 0 )
        self.sel_l = gr.multiply_const_ff( 1 )
        self.sum   = gr.add_ff( )

                
        fg.connect(   self.xlate,          self.split      )
        fg.connect( ( self.split, 0 ),     self.lpf_i      )
        fg.connect( ( self.split, 1 ),     self.lpf_q      )
        fg.connect(   self.lpf_i,        ( self.sum_l, 0 ) )
        fg.connect(   self.lpf_q,        ( self.sum_l, 1 ) )
        fg.connect(   self.lpf_i,        ( self.sum_u, 0 ) )
        fg.connect(   self.lpf_q,        ( self.sum_u, 1 ) )
        fg.connect(   self.sum_l,          self.sel_l,     )
        fg.connect(   self.sum_u,          self.sel_u,     )
        fg.connect(   self.sel_l,        ( self.sum,   0 ) )
        fg.connect(   self.sel_u,        ( self.sum,   1 ) )
        
        gr.hier_block.__init__( self, fg, self.xlate, self.sum )

    def upper_sb( self ):
        if self.sideband != -1:
            self.sideband = -1
            self.sel_l.set_k( 0.0 )
            self.sel_u.set_k( 1.0 )
        
    def lower_sb( self ):
        if self.sideband != 1:
            self.sideband = 1
            self.sel_l.set_k( 1.0 )
            self.sel_u.set_k( 0.0 )

    def tune( self, freq ):
        self.xlate.set_center_freq( freq )

